# -*- coding: utf-8 -*-
"""
Created on Tue Oct 20 20:20:20 2020
@authors: Amal Htait and Leif Azzopardi
For license information, see LICENSE.TXT
"""

from awessome.awessome_builder import *

if __name__ == "__main__":

    avg_builder = SentimentIntensityScorerBuilder('avg', 'bert-base-nli-mean-tokens', 'euclidean', '100', True)
    max_builder = SentimentIntensityScorerBuilder('max', 'bert-base-nli-mean-tokens')
    vader_avg_scorer = avg_builder.build_scorer_from_prebuilt_lexicon('vader')
    labmt_avg_scorer = avg_builder.build_scorer_from_prebuilt_lexicon('labmt')
    vader_max_scorer = max_builder.build_scorer_from_prebuilt_lexicon('vader')
    labmt_max_scorer = max_builder.build_scorer_from_prebuilt_lexicon('labmt')

    sentences = ['I am happy',
                'I am very happy',
                'I am not happy',
                'I am feeling very sad',
                'I cant wait to go to the party',
                'This is awessome!!',
                'This is awesome!!',
                'This is awesome',
                'lol',
                ':-)']

    for sentence in sentences:
        print(sentence)
        print(vader_avg_scorer.name, vader_avg_scorer.score_sentence(sentence))
        print(labmt_avg_scorer.name, labmt_avg_scorer.score_sentence(sentence))
        print(vader_max_scorer.name, vader_max_scorer.score_sentence(sentence))
        print(labmt_max_scorer.name, labmt_max_scorer.score_sentence(sentence))
        print('------------------------------------------------------------------------')



# -*- coding: utf-8 -*-
"""
Created on Tue Oct 20 20:20:20 2020
@authors: Amal Htait and Leif Azzopardi
For license information, see LICENSE.TXT
"""

import scipy 


class Similarity(object):
    """
    Apply a similarity measure between a text embedding and seeds lists embeddings: cosine or  euclidean.
    """
    def __init__(self):
        pass
        
    def score(self, text_embedding, seeds_embeddings):
        """
        Return the similarity distance between embeddings using scipy.

        :param text_embedding: an embedding representation for the text using the language model.
        :param seeds_embeddings: a list of embeddings representing the seed words using the language model
        :returns: a similarity (float)
        """
        return 0.0

    
class CosineSimilarity(Similarity):
    """
    Apply a cosine similarity measure between a text embedding and seeds lists embeddings.
    """
    def score(self, text_embedding, seeds_embeddings):
        sim_score = scipy.spatial.distance.cdist([text_embedding], seeds_embeddings, 'cosine')[0]
               
        return sim_score


class EuclideanSimilarity(Similarity):
    """
    Apply a euclidean similarity measure between a text embedding and seeds lists embeddings.
    """
    def score(self, text_embedding, seeds_embeddings):
        sim_score = scipy.spatial.distance.cdist([text_embedding], seeds_embeddings, 'euclidean')[0]

        return sim_score



# -*- coding: utf-8 -*-
"""
Created on Tue Oct 20 20:20:20 2020
@authors: Amal Htait and Leif Azzopardi
For license information, see LICENSE.TXT
"""


class SentimentIntensityAggregator:
    """
    Apply an aggregation to a list of similarity scores.
    """

    def __init__(self):
        pass

    def agg_score(self, pos_dists, neg_dists):
        """
        Return a float as a score of aggregated similarity scores with positive and negative seeds lists.

        :param pos_dists: a list of similarity scores with pos seeds
        :param neg_dists: a list of similarity scores with neg seeds
        :return: a score (float)
        """
        pass


class SumSentimentIntensityAggregator(SentimentIntensityAggregator):
    """
    Apply an sum aggregation to a list of similarity scores.
    """
    def agg_score(self, pos_dists, neg_dists):
        pos_score=0.0
        neg_score=0.0
        dists_size = len(pos_dists)
        for i in range(dists_size):
            pos_score=pos_score + (1.0-pos_dists[i])
            neg_score=neg_score + (1.0-neg_dists[i])
        score=pos_score - neg_score
               
        return score


class AvgSentimentIntensityAggregator(SentimentIntensityAggregator):
    """
    Apply an average aggregation to a list of similarity scores.
    """
    def agg_score(self, pos_dists, neg_dists):
        pos_sum = 0.0
        neg_sum = 0.0
        dists_size = len(pos_dists)
        for i in range(dists_size):
            pos_sum = pos_sum + (1.0-pos_dists[i])
            neg_sum = neg_sum + (1.0-neg_dists[i])
        pos_score = pos_sum/dists_size
        neg_score = neg_sum/dists_size
        score = pos_score - neg_score
        
        return score
        

class MaxSentimentIntensityAggregator(SentimentIntensityAggregator):
    """
    Selecting the maximum value in a list of similarity scores.
    """
    def agg_score(self, pos_dists, neg_dists):
        pos_score = (1.0-(min(pos_dists)))
        neg_score = (1.0-(min(neg_dists)))
        score = pos_score - neg_score
        
        return score



# -*- coding: utf-8 -*-
"""
Created on Tue Oct 20 20:20:20 2020
@authors: Amal Htait and Leif Azzopardi
For license information, see LICENSE.TXT
"""


class SentimentIntensityScorer(object):
    """
    Give a sentiment intensity score to sentences.
    """

    def __init__(self, pos_seeds_embeddings, neg_seeds_embeddings, aggregator,  language_model_embedder, similarity, weighted, lexicon_list, lexicon_dict):
        """

        :param pos_seeds_embeddings: The word embedding representation of positive seed words from awessome.awessome_builder
        :param neg_seeds_embeddings: The word embedding representation of negative seed words from awessome.awessome_builder
        :param aggregator: expects an aggregation object from awessome.awessome_aggregator
        :param language_model_embedder: The embedder created based on pre-trained model, provided by awessome.awessome_builder
        :param similarity: expects a similarity object from awessome.awessome_similarity
        :param weighted: is a boolean, if True we use a dictionary of terms as weights to modify the similarity between 
        """
        self.pos_seeds_embeddings = pos_seeds_embeddings
        self.neg_seeds_embeddings = neg_seeds_embeddings
        self.aggregator = aggregator
        self.similarity = similarity
        self.language_model_embedder = language_model_embedder
        self.weighted = weighted
        self.lexicon_list = lexicon_list
        self.lexicon_dict = lexicon_dict
        self.name = 'awessome'

    def score_sentence(self, text):
        """
        Return the sentiment strength of the input text.
        Positive values are positive scores, negative value are negative
        scores.

        :param text: The sentence to predict its sentiment intensity
        :return: a float for sentiment strength
        """
        text = self.split_long_sentence(text)
        text_embedding = self.language_model_embedder.encode(text)

        distances_pos = self.similarity.score(text_embedding, self.pos_seeds_embeddings)
        distances_neg = self.similarity.score(text_embedding, self.neg_seeds_embeddings)

        if self.weighted:
            distances_pos, distances_neg = self.add_weight(distances_pos, distances_neg)

        score = self.aggregator.agg_score(distances_pos, distances_neg)

        return score

    def score_list(self, sentence_list):
        """
        Return the sentiment strength of the input list of sentences.

        :param sentence_list: The list of sentences to predict their sentiment intensity.
        :return: a dictionary of sentence and sentiment strength (float)
        """
        score_dict = {}
        for i in range(len(sentence_list)):
            score = self.score_sentence(sentence_list[i])
            score_dict[sentence_list[i]] = score

        return score_dict

    def split_long_sentence(self, text):
        """
        Limit the input text to 256 only,
        due to pre-trained language models input limitation

        :param text: The sentence to predict its sentiment intensity
        :return: new shorter text
        """
        max_length = 256
        separator = ' '
        text_length = len(text.split())

        if text_length > max_length:
            text_list = text.split()[:max_length]
            text = separator.join(text_list)

        return text 

    def add_weight(self, distances_pos, distances_neg):
        """
        Add a weight value to the distance similarity score between the sentence and the seeds lists.

        :param distances_pos: The distance similarity score between the sentence and the positive seeds lists, via awessome.awessome_similarity.
        :param distances_neg: The distance similarity score between the sentence and the negative seeds lists, via awessome.awessome_similarity.
        :return: New distance similarity scores with weight application.
        """
        distances_pos_w = []
        distances_neg_w = [] 

        for i in range(len(distances_pos)):
            distances_pos_w.append(distances_pos[i] * float(self.lexicon_dict[self.lexicon_list[i]]))
            distances_neg_w.append(distances_neg[i] * float(self.lexicon_dict[self.lexicon_list[i]]))

        return distances_pos_w, distances_neg_w




# -*- coding: utf-8 -*-
"""
Created on Tue Oct 20 20:20:20 2020
@authors: Amal Htait and Leif Azzopardi
For license information, see LICENSE.TXT
"""

import operator
import os

from awessome.awessome_aggregator import SumSentimentIntensityAggregator, AvgSentimentIntensityAggregator, \
   MaxSentimentIntensityAggregator
from awessome.awessome_scorer import SentimentIntensityScorer
from awessome.awessome_similarity import CosineSimilarity, EuclideanSimilarity
from sentence_transformers import SentenceTransformer

# Default values
DEFAULT_SEED_SIZE =500
DEFAULT_AGGREGATOR ='avg'
DEFAULT_LEXICON = 'vader'
DEFAULT_LANGUAGE_MODEL = 'bert-base-nli-mean-tokens'
DEFAULT_SIMILARITY_METHOD = 'cosine'
DEFAULT_WEIGHTED = False


class SentimentIntensityScorerBuilder(object):
    """
    Build the sentiment intensity scorer
    """

    def __init__(self, aggregation_method_name, language_model_name, similarity_method_name=DEFAULT_SIMILARITY_METHOD, seed_size=DEFAULT_SEED_SIZE, weighted=DEFAULT_WEIGHTED):
        """

        :param aggregation_method_name: Selected aggregation method : avg, sum or max (default = avg)
        :param language_model_name: Selected pre-trained language model: bert-base-nli-mean-tokens, distilbert-base-nli-stsb-mean-tokens, etc
                (default = bert-base-nli-mean-tokens)
        :param similarity_method_name: Selected similarity method : cosine or euclidean (default = cosine)
        :param seed_size: Selected seeds lists size (default = 500)
        :param weighted: Option of adding weight to aggregated similarity scores in the sentiment (default = False)
        """
        self.aggregator_method_name = aggregation_method_name
        self.language_model_name = language_model_name
        self.similarity_method_name = similarity_method_name
        self.seed_size=seed_size
        self.weighted=weighted
        self.lexicon_name=None
        self.aggregator = self._create_aggregator(aggregation_method_name)
        self.similarity_method = self._create_similarity(similarity_method_name)
        self.language_model  = self._create_language_model(language_model_name)

    def build_scorer_from_lexicon_file(self, lexicon_file):
        """
        Build and return a scorer object built using all the given parameters, including an input lexicon file

        :param lexicon_file:
        :return: Sentiment Intensity Scorer Object
        """
        filename = os.path.basename(lexicon_file)
        lexicon_list, lexicon_dict = self._load_lexicon_from_file(lexicon_file)
        pos_seeds_embeddings, neg_seeds_embeddings = self._make_seed_lists(lexicon_list, self.language_model, self.seed_size)

        scorer = SentimentIntensityScorer(pos_seeds_embeddings=pos_seeds_embeddings, neg_seeds_embeddings=neg_seeds_embeddings,
                                                        aggregator=self.aggregator, language_model_embedder=self.language_model, similarity=self.similarity_method, weighted=self.weighted, lexicon_list=lexicon_list, lexicon_dict=lexicon_dict)

        scorer.name = 'awessome-{}-{}-{}-{}-{}'.format(self.aggregator_method_name, self.language_model_name, self.seed_size, self.similarity_method_name, filename)
        return scorer

    def build_scorer_from_prebuilt_lexicon(self, lexicon):
        """
        Build and return a scorer object built using all the given parameters, using a prebuilt lexicon
        by calling the function: build_scorer_from_lexicon_file

        :param lexicon: the name of a prebuilt lexicon (string)
        :param seed_size: the number of positive and negative seed terms to use given the lexicon
        :return: SentimentIntensityScorer
        """
        self.lexicon_name = lexicon

        lexicons = ['vader',
                          'labmt']

        if lexicon not in lexicons:
            lexicon = DEFAULT_LEXICON

        lexicon_file  =  'lexicon/{}.txt'.format(lexicon)
        #lexicon_file  =  'lexicon/{}.lex'.format(lexicon) => we'll need to change our lexicon files to .lex

        return self.build_scorer_from_lexicon_file(lexicon_file)

    def _load_lexicon_from_file(self, lexicon_file):
        """
        Load the lexicon to be used in the scorer from an input file.

        :param lexicon_file: the name of the file containing seed terms and their
                sentiment intensity score (tab separated file).
        :return: the list of lexicon terms, and a dictionary of terms (key) and SI scores (value).
        """
        lexicon_dict = {}
        seeds = open(lexicon_file, 'r', encoding='utf-8-sig').readlines()
        for i in range(len(seeds)):
            line = seeds[i].split('\t')
            lexicon_dict[line[0].strip()] = line[1].strip()
            lexicon_dict = dict( sorted( lexicon_dict.items(), key=operator.itemgetter(1), reverse=True ) )

        return list(lexicon_dict.keys()), lexicon_dict

    def _create_aggregator(self, aggregation_method):
        """
        Return the aggregation class to use based on an input parameter.

        :param aggregation_method: Aggregation method that can be : avg, sum or max.
        :return: Aggregation class to use.
        """

        aggregators={
            'sum': SumSentimentIntensityAggregator,
            'avg': AvgSentimentIntensityAggregator,
            'max': MaxSentimentIntensityAggregator
         }

        am = aggregation_method.lower()
        if am in aggregators:
              aggregator = aggregators[am]()
        else:
            aggregator=aggregators[DEFAULT_AGGREGATOR]()
            self.aggregator_method_name = DEFAULT_AGGREGATOR

        return aggregator

    def _create_language_model(self, language_model_name):
        """
        Return a created embedder based on an input parameter.

        :param language_model_name: pretrained language model name (ex: bert-base-nli-mean-tokens).
        :return: the embedder created using the language model, 
                   that will be used to create the embedding representation of text.
        """

        models_name = [
            'bert-base-nli-mean-tokens', #fast
            'bert-large-nli-mean-tokens', 
            'bert-base-nli-stsb-mean-tokens', 
            'bert-large-nli-stsb-mean-tokens',
            #'roberta-base-nli-stsb-mean-tokens', #TypeError: __init__() got an unexpected keyword argument 'do_lower_case'
            #'roberta-large-nli-stsb-mean-tokens', #TypeError: __init__() got an unexpected keyword argument 'do_lower_case'
            'distilbert-base-nli-stsb-mean-tokens', # needs GPU
            'xlm-r-100langs-bert-base-nli-stsb-mean-tokens', # needs GPU
            'xlm-r-100langs-bert-base-nli-mean-tokens' # needs GPU
        ]

        if language_model_name in models_name:
            language_model_embedder = SentenceTransformer(language_model_name)
        else:
            language_model_embedder = SentenceTransformer(DEFAULT_LANGUAGE_MODEL)
            self.language_model_name = DEFAULT_LANGUAGE_MODEL

        return language_model_embedder

    def _make_seed_lists(self, lexicon_list, language_model_embedder, seed_size):
        """
        Return two embedding lists representing the seeds lists (positive and negative)

        :param lexicon_list: a lexicon list extracted from the lexicon file (from _create_language_model)
        :param language_model_embedder: embedder created based on the pretrained language model
                (from_create_language_model)
        :param seed_size: size of seeds lists (default = 500)
        :return: Two embedding lists representing the seeds lists
        """

        max_seed_size = len(lexicon_list)/2
        if int(seed_size) > max_seed_size:
            seed_size=max_seed_size

        pos_seeds=lexicon_list[:int(seed_size)]
        neg_seeds=lexicon_list[-int(seed_size):]

        pos_seeds_embeddings=language_model_embedder.encode(pos_seeds)
        neg_seeds_embeddings=language_model_embedder.encode(neg_seeds)

        return pos_seeds_embeddings, neg_seeds_embeddings

    def _create_similarity(self, similarity_method):
        """
        Return the Class to use for the input similarity method

        :param similarity_method: 'cosine','euclidean'
        :return: a Similarity scoring object
        """

        similarity_methods = {
            'cosine': CosineSimilarity,
            'euclidean': EuclideanSimilarity
        }

        sm = similarity_method.lower()
        if sm in similarity_methods:
            sim = similarity_methods[sm]()
        else:
            sim = similarity_methods[DEFAULT_SIMILARITY_METHOD]()
            self.similarity_method_name = DEFAULT_SIMILARITY_METHOD

        return sim







from setuptools import setup, find_packages
import runpy

PACKAGE_NAME = "awessome"
#version_meta = runpy.run_path("./version.py")
#VERSION = version_meta["__version__"]

classifiers = ['Development Status :: 4 - Beta', 'Intended Audience :: Science/Research',
                 'License :: OSI Approved :: MIT License', 'Natural Language :: English',
                 'Programming Language :: Python :: 3.5', 'Topic :: Scientific/Engineering :: Artificial Intelligence',
                 'Topic :: Scientific/Engineering :: Information Analysis', 'Topic :: Text Processing :: Linguistic',
                 'Topic :: Text Processing :: General']

def parse_requirements(filename):
        """Load requirements from a pip requirements file."""
        lineiter = (line.strip() for line in open(filename))
        return [line for line in lineiter if line and not line.startswith("#")]


setup(
  name=PACKAGE_NAME,
  version='0.0.1',
  description='awessome',
  include_package_data=True,
  install_requires=parse_requirements("requirements.txt"),
  long_description=open('README.txt').read() + '\n\n' + open('CHANGELOG.txt').read(),
  url='https://github.com/cumulative-revelations/awessome',  
  author='Amal Htait and Leif Azzopardi',
  author_email='amal.htait@strath.ac.uk, leif.azzopardi@strath.ac.uk',
  license='MIT', 
  classifiers=classifiers,
  keywords=['awessome', 'sentiment', 'analysis', 'opinion', 'mining', 'nlp', 'text', 'data',
            'text analysis', 'opinion analysis', 'sentiment analysis', 'text mining', 'twitter sentiment',
            'opinion mining', 'social media', 'twitter', 'social', 'media'], 
  packages=find_packages()
)
